 <?php
/******************************************************/
/***												***/
/***			Lydkode() Revision 2.0.3			***/
/***												***/
/***			From a original idea by				***/
/***				Troels S Eriksen				***/
/***												***/
/***			Rewritten to PHP by					***/
/***		Peter Falk (pf@typoconsult.dk)			***/
/***				TypoConsult						***/
/***					2010						***/
/***												***/
/******************************************************/

/*
 * To convert a string to a phonetic code:
 *
 * Usage:
 * 		$phonetic_values = phonetic_search::phonetic($my_string, $chars = 6);
 * $chars is the length of the string to encode - max is 8 characters, default is 6
 *
 * $phonetic_values = array (
 * 		'primary',	# the primary sound code
 * 		'secondary,	# the secondary sound code
 * );
 *
 * Be aware that this implementation is made for TYPO3.
 * It takes advantage of two TYPO3 functions from the library t3lib_div:
 * - inList ~ determines if a instance is a member of a list, i.e. is the letter in the list of vowels
 * - danish_toupper ~ converts all the characters to upper case
 *
 * To use this class you must replace these two function with similar ones
 *
 */


require_once(PATH_t3lib.'class.t3lib_div.php');

class phonetic_search {

	private static $vowels = array('A', 'E', 'I', 'O', 'U', 'Y', 'Ä', 'Ü', 'Ö', 'Æ', 'Ø', 'Å');
	private static $charValues = array(
		'F' => 1,
		'G' => 2,
		'H' => 3,
		'J' => 4,
		'K' => 5,
		'L'	=> 6,
		'M' => 7,
		'N' => 8,
		'P' => 9,
		'R' => 10,
		'S' => 11,
		'T' => 12,
		'V' => 13,
		'X' => 14,
		'A' => 15,
		//'other' => 0,
	);

	/**
	 * The main function
	 * @param $in_string
	 *
	 * @return array
	 */
	public static function phonetic($in_string, $chars = 6) {
		$in_string = self::str2upper($in_string);
		$codes = self::soundCode($in_string);

		return array(
			'primary' => self::calcIntValue($codes[0], $chars),
			'secondary' => self::calcIntValue($codes[1], $chars),
		);
	}

 
	/**
	 * Determines if character is a vowel
	 * @param $chr
	 *
	 * @return boolean
	 */
	private function isVowel($chr) {
		return t3lib_div::inList(implode(',',self::$vowels), $chr);
	}

	/**
	 * Calculates integer key for each search word
	 * @param string $str
	 *
	 * @return int
	 */
	private function calcIntValue($str, $len) {

		$i = 0;
		$j = $len > 8 ? 28 : $len * 4;
		while( $str[$i] && $i < $len) {
			$chrVal =  isset(self::$charValues[$str[$i]]) ? self::$charValues[$str[$i]] : 0;
			$r = $r | sprintf("%u", ($chrVal << $j));
			$i++;
			$j -= 4;
		}

		return $r == 0 ? NULL : $r;
	}

	/**
	 * Converts string to uppercase
	 * @param string $str
	 * @return string converted to uppercase
	 */
	private function str2upper($str) {
		$source = array('ä', 'ü', 'ö');
		$replace = array('Ä', 'Ü', 'Ö');

		$str = t3lib_div::danish_strtoupper($str);
		$str = str_replace($source, $replace, $str);

		return $str;
	}


	/**
	 * Reduces the string to a phonetic code
	 * @param string $str
	 *
	 * @return array - primary key and secondary key
	 */
	private function soundCode($str) {
		// Remove spaces and accents
		$org = str_replace(array(' ',"'"), '', $str);

		// exceptions
		$org = str_replace(array('AV', 'OV'), array('AU', 'OU'), $org);
		// Take care of DATTER, DOTTIR, KONE and DTR
		$org = preg_replace('/(.*)(D[OA]TT[IE]R|KONE|DTR)$/i', '$1SEN', $org); 

		$len = strlen($org);
  		$c = 0;
  		$s = '';
  		$p = '';

  		if(strncmp($org, 'SCHM', 4) === 0) {
  			$p .= 'S';
    		$s .= 'S';
    		$c += 3;
  		}

  		elseif ($org[0] == 'X') {
    		$p .= 'S';
    		$s .= 'S';
    		$c += 1;
  		}

  		elseif(t3lib_div::inList(array('GN','HV','KN','PS','WR'), substr($org, 0, 2))) {
  			$c += 1;
  		}

  		elseif (self::isVowel($org[0])) {
  			$p .= 'A';
  			$s .= 'A';
  			if(strcmp($org,'OUE') === 0) {
  				$org = 'OVE';
  			}
  			$c += 1;
  		}

  		while ($c < $len) {
  			if($c > 0) {
  				$prv = $org[$c-1];
  			}
  			$cur = $org[$c];
  			$nxt = $org[$c+1];


  			switch($cur) {
  				case 'B':
  					$p .= 'P';
  					$s .= 'P';
  					break;
  				case 'C':
		  			if($nxt == 'K') {
		  				//nothing (like Mickey, Rickey - added by Peter Falk)
		  				// May be the condition should apply to K
		  			} else {
		  				$p .= 'K'; // som Claus, Michael
		  			}

		  			if($prv == 'S' && $nxt == 'H') { // SCHØTT matcher SJØTT
		  				$s .= 'J';
		  			} elseif ($nxt == 'H' || $nxt == 'L') {
		  				$s .= 'K';
		  			} else {
		  				$s .= 'S';
		  			}

		  			if($nxt == 'H' || $nxt == 'P') {
		  				$c += 1;
		  			}
  					break;
  				case 'D':
	  				if($prv == 'N' || $prv == 'L') { // D efter N og L kan ikke høres
	  					// nothing
	  				} else {
	  					$p .= 'T';
	  					$s .= 'T';
	  				}

	  				if($nxt == 'T') {
	  					$c += 1;
	  				}
	  				break;
  				case 'F':
  					$p .= 'F';
  					$s .= 'F';
  					break;
  				case 'G':
  					$p .= 'G';
  					$s .= 'K';
  					break;
  				case 'H':
  					if($nxt != 'J' && $nxt != 'V') { // HJ giver kun J lyd, HV kun V lyd
	  					$p .= 'H';
	  				}

	  				if($c == 0 && self::isVowel($nxt)) {
	  					$s .= 'A'; // Stumt H i start ELLEN matcher HELLEN
	  				}

  					break;
  				case 'I':
  				case 'Y':
		  			if(self::isVowel($prv) || self::isVowel($nxt)) {
	  					$s .= 'J';
	  				}
  					break;
  				case 'J':
	  				if($prv == 'P' || $prv == 'K') { // J ignores efter P og K - så bliver PJOTR lig PETER og KJELL matcher KELD
	  				} else {
	  					$p .= 'J';
	  				}

	  				if ($c == $len && self::isVowel($prv)) { // efter vokal I udtales J - fx NICOLAI - så vi undertrykker J'et
	  				} else {
	  					$s .= 'J';
	  				}
  					break;
  				case 'K':
  					$p .= 'K';
      				$s .= 'K';
  					break;
  				case 'L':
  					$p .= 'L';
      				$s .= 'L';
  					break;
  				case 'M':
  					$p .= 'M';
      				$s .= 'M';
  					break;
  				case 'N':
		  			$p .= 'N';
					if($c == 0 && substr($org, 0, 3) == 'NIS') { // NIS = NIELS
						$s .= 'NL';
					} else {
						$s .= 'N';
					}
  					break;
  				case 'P':
	  				if ($nxt == 'H') {
		        		$p .= 'F';
		        		$s .= 'P';
		        		$c += 1;
	      			} else {
		        		$p .= 'P';
		        		$s .= 'P';
		        	}
	  				break;
  				case 'Q':
  					$p .= 'K';
        			$s .= 'K';
  					break;
  				case 'R':
	  				$p .= 'R';
					if ($prv == 'A' && $nxt == 'S') { //  ' Match Lars med Lasse
					} else {
						$s .= 'R';
					}
					break;
  				case 'S':
	  				$p .= 'S';
					$s .= 'S';
					if (strpos($org, "SCHL", $c) > 0) {
						$c += 2;   //' SCHL udtales normalt SL
					}
					if ($nxt == 'Z') {
						$c += 1; 		//' SZ opfattes som SS
					}
	  				break;
  				case 'T':
	  				$p .= 'T';
	      			$s .= 'T';
	      			if ($nxt == 'H') {
	      				$c += 1;
	      			}
	      			break;
  				case 'V':
	  				$p .= 'V';
					if (self::isVowel($prv)) {  // V udtales enten som V eller F - men kun efter vokal.
						$s .= 'F';
					} else {
						$s .= 'V';
					}
  					break;
  				case 'W':
  					$p .= 'V';	// W udtales som V - men IWERSEN skal også matche IFERSEN
      				$s .= 'F';
  					break;
  				case 'X':
  					$p .= 'KS';
					$s .= 'C';
  					break;
  				case 'Z':
	  				$p .= 'S';
	      			$s .= 'S';
	      			if ($nxt == 'S') {
	      				$c += 1;
	      			}
  					break;
  			}

  			$c += 1;
  			if ($nxt == $cur) {
  				$c += 1;
  			}
  		}

  		if($s == $p) {
  			$s = '';
  		}

  		return array($p, $s);
	}
}

//var_dump(phonetic_search::phonetic('pheodora'));
//var_dump(phonetic_search::phonetic('petra'));
//var_dump(phonetic_search::phonetic('badr'));

?>